/* Terraform - (C) 1997-1999 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFDialogMirror.h"
#include "GuiDialogOAC.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"


/*
 *  constructor: initialize all data members 
 */
TFDialogMirror::TFDialogMirror (HeightField *HF, HeightFieldDraw *HFD)
	        : TFPreviewDialog (HF, HFD, "Terraform Mirror Dialog", 
				"Mirror along which Axis", 6, 4),
		  d_vBoxOptions (TRUE, 5),
		  d_optionsFrame ("Mirror Type")
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogMirror\n");
	d_RB1 = d_RB2 = d_RB3 = d_RB4 = NULL;
	d_group = NULL;
	buildDialogWindow ();
	this->setHFobjs (HF, HFD);
	sprintf (this->d_winTitle, "Mirror: %s", d_HF->getName());
	this->set_title (this->d_winTitle);
	this->iterateEvents ();
	updatePreviewCallback ();

	connect_to_method (d_RB1->clicked, this, &TFDialogMirror::updatePreviewCallback);
	connect_to_method (d_RB2->clicked, this, &TFDialogMirror::updatePreviewCallback);
	connect_to_method (d_RB3->clicked, this, &TFDialogMirror::updatePreviewCallback);
	connect_to_method (d_RB4->clicked, this, &TFDialogMirror::updatePreviewCallback);

}


/*
 *  destructor: clean up 
 */
TFDialogMirror::~TFDialogMirror ()
{
	if (d_RB1) delete d_RB1;
	if (d_RB2) delete d_RB2;
	if (d_RB3) delete d_RB3;
	if (d_RB4) delete d_RB4;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogMirror\n");
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogMirror::insertOptions()
{
	d_optionsFrame.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_optionsFrame.set_border_width (5);

	d_RB1 = new Gtk_RadioButton ((GSList *)NULL, "Horizonal Axis (-)");
	d_RB1->set_active(TRUE);
	d_vBoxOptions.pack_start (*d_RB1, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_RB1->show ();

	d_group = d_RB1->group ();
	d_RB2 = new Gtk_RadioButton (d_group, "Vertical Axis (|)");
	d_vBoxOptions.pack_start (*d_RB2, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_RB2->show ();

	d_group = d_RB2->group ();
	d_RB3 = new Gtk_RadioButton (d_group, "Top Left - Bottom Right (\\)");
	d_vBoxOptions.pack_start (*d_RB3, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_RB3->show ();

	d_group = d_RB3->group ();
	d_RB4 = new Gtk_RadioButton (d_group, "Bottom Left - Top Right (/)");
	d_vBoxOptions.pack_start (*d_RB4, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_RB4->show ();

	d_optionsFrame.add (d_vBoxOptions);
	this->d_table->attach (d_optionsFrame, 3, 6, 1, 3);
	d_vBoxOptions.show ();
	d_optionsFrame.show ();
	this->d_baseFrame->show ();
	this->get_vbox()->show ();
}


/*
 *  buildDialogWindow:
 */
void TFDialogMirror::buildDialogWindow ()
{
	this->set_usize (375, 225);
	this->set_title (this->d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	this->insertPreview();
	insertOptions ();
	this->fillActionArea ();

	this->show ();
}


/*
 *  buttonCallbackApply: rotate the Height Field
 */
void TFDialogMirror::buttonCallbackApply ()
{
	SanityCheck::bailout ((!d_HFO), "d_HFO==NULL", "TFDialogMirror::buttonCallbackApply");
	SanityCheck::bailout ((!d_HFD), "d_HFD==NULL", "TFDialogMirror::buttonCallbackApply");

	if (d_RB1->get_active())
		d_HFO->mirror (1);
	else
	if (d_RB2->get_active())
		d_HFO->mirror (2);
	else
	if (d_RB3->get_active())
		d_HFO->mirror (3);
	else
	if (d_RB4->get_active())
		d_HFO->mirror (4);
	d_HFD->draw ();

	// only do this if window stays open 
	if (d_applyHit)
		{
		this->setHFobjs (d_HF, d_HFD);
		updatePreviewCallback ();
		}

	this->d_applyHit = TRUE;
}


/*
 *  updatePreviewCallback: update the preview after a widget has been updated.
 */
void TFDialogMirror::updatePreviewCallback ()
{
	if (!d_cbUsePreview.get_active())
		return;

	SanityCheck::bailout ((!d_HFPreview), "d_HFPreview==NULL", "TFDialogFill");
	SanityCheck::bailout ((!d_HFOPreview), "d_HFOPreview==NULL", "TFDialogFill");
	SanityCheck::bailout ((!d_HFDPreview), "d_HFDPreview==NULL", "TFDialogFill");
	this->previewUpToDate ();
	d_HFPreview->restore ();
	if (d_RB1->get_active())
		d_HFOPreview->mirror (1);
	else
	if (d_RB2->get_active())
		d_HFOPreview->mirror (2);
	else
	if (d_RB3->get_active())
		d_HFOPreview->mirror (3);
	else
	if (d_RB4->get_active())
		d_HFOPreview->mirror (4);
	d_HFDPreview->setColormap (d_HFD->getColormap());
	d_HFDPreview->draw ();
}

